@@ -127,6 +127,14 @@ void              gp_error_fatal() ;
 
 void              gp_codon_init_conversion() ;
 int               gp_codon_isstart(sekw *s, long p) ;
+int								gp_codon_load_usage(FILE *fp, double utable[64]) ;
+int								gp_codon_load_code(FILE *fp, int ctable[64]) ;
+void							gp_codon_load_code_standard(int ctable[64]) ;
+int								gp_codon_get_index(char *codon) ;
+int								gp_codon_get_codon(char *codon, int index) ;
+int								gp_codon_get_usage(sekw *s, double cusage[64]) ;
+
+/* depreciated */
 codont*           gp_codon_table_load_std() ;
 
 void              gp_seq_free(sekw *s) ;
@@ -881,8 +889,7 @@ sekw* gp_seq_dna_to_protein(sekw *inseq, codont* intable, long start, int pedant
 	 * written in the sequence outseq */
 
 	long dlugosc,i = 0,j = 0,aaread = 0 ;
-	char name[MAXNAZWA] ;
-	char t, *tmp;
+	char t ;
 	sekw *outseq ;
 
 	/* 
@@ -893,31 +900,27 @@ sekw* gp_seq_dna_to_protein(sekw *inseq, codont* intable, long start, int pedant
 	for(i = 0;i<128;i++) Conv[i] = 99 ;
 	Conv['A'] = 0 ; Conv['C'] = 1; Conv['G'] = 2 ; Conv['T'] = 3 ; Conv['U'] = 3 ;
 
-	if(debug) gp_warn("Dna2Protein: from %i, pedantic = %i",start,pedantic) ;
+	if(debug) gp_warn("gp_seq_dna_to_protein: from %i, pedantic = %i",start,pedantic) ;
 
-	if((tmp = strchr(inseq->name,'(')) != NULL) tmp[0] = '\0' ;
-	if(strlen(inseq->name)>= MAXNAZWA)
-		inseq->name[MAXNAZWA-12] = '\0' ; 
-
-	strcpy(name,inseq->name) ;
-	strcat(name, " (translated)") ;
 	/* Allocating enough memory for the outseq */
 
 	dlugosc = inseq->leng ;
 	outseq = (sekw*) calloc(1,sizeof(sekw)) ;
 	outseq->sequ = (char*) calloc((dlugosc/3 + 2),sizeof(char)) ;
 	outseq->type = PROTEIN ;
-	strcpy(outseq->name,name) ;
+	outseq->name = gp_strdup(inseq->name) ;
+
+	if(start == 0) start = 1 ;
 
 	/* first triplet: is it a valid start codon? */
 	for(i = 0, j = start - 1 ; i < 3 ; i++, j++) 
-		coord[i] = Conv[inseq->sequ[j]] ;
+		coord[i] = Conv[(int) inseq->sequ[j]] ;
 
 	if(pedantic && (gp_codon_isstart(inseq,start-1) == FALSE)) {
 		if(debug)
-			gp_warn("D2P: %s does not start with a start codon!", inseq->name) ;
-		free(outseq->sequ) ;
-		free(outseq) ;
+			gp_warn("gp_seq_dna_to_protein: %s does not start with a start codon!", 
+				inseq->name) ;
+		gp_seq_free(outseq) ;
 		return(NULL) ;
 	}
 
@@ -930,7 +933,7 @@ sekw* gp_seq_dna_to_protein(sekw *inseq, codont* intable, long start, int pedant
 		/* getting the next triplet. Watch out for the end of the sequence */
 		for(i = 0;i<3;i++,j++) {
 			if(j == dlugosc) check = FALSE ;
-			else coord[i] = Conv[inseq->sequ[j]] ;
+			else coord[i] = Conv[(int) (inseq->sequ[j])] ;
 		}
 
 		if(check == TRUE) t = intable->tbl[coord[0]][coord[1]][coord[2]] ;
@@ -938,25 +941,192 @@ sekw* gp_seq_dna_to_protein(sekw *inseq, codont* intable, long start, int pedant
 	
 	if(pedantic && check == FALSE) {
 		if(debug)
-			gp_warn("D2P: No stop codon found in %s",inseq->name) ;
-		free(outseq->sequ) ;
-		free(outseq) ;
+			gp_warn("gp_seq_dna_to_protein: No stop codon found in %s",inseq->name) ;
+		gp_seq_free(outseq) ;
 		return(NULL) ;
 	}
 
 	outseq->sequ[aaread] = '\0' ;
 	outseq->leng = aaread ;
 
-	if(debug) gp_warn("Dna2Prot: %s translated succesfully, %i residues",
+	if(debug) gp_warn("gp_seq_dna_to_protein: %s translated succesfully, %i residues",
 		inseq->name,strlen(outseq->sequ)) ;
 	return(outseq) ;
 
 }
 
 
+/* get the codon usage of a sequence */
+int gp_codon_get_usage(sekw *s, double cusage[64]) {
+	int i, index, aa_num, aa_toto ;
+	char *pos ;
+
+	aa_num = s->leng / 3 ;
+	pos = s->sequ ;
+	if(debug) gp_warn("gp_codon_get_usage: seq %s, l %i, aa %i", s->name, s->leng, aa_num) ;
+
+	/* zero the table */
+	for(i = 0 ; i < 64 ; i++) cusage[i] = 0.0 ;
+
+	aa_toto = 0 ;
+
+	/* collect codon numbers */
+	for(i = 0 ; i < aa_num ; i++, pos += 3) {
+		index = gp_codon_get_index(pos) ;
+		if(index < 0 || index > 64) {
+			gp_warn("gp_codon_get_usage: problems reading a codon-index %i", index) ;
+		} else {
+			cusage[index] += 1.0 ;
+			aa_toto++ ;
+		}
+	}
+
+	/* go for the frequencies -- only if total num of aa is > 0 ! */
+	if(aa_toto != 0) for(i = 0 ; i < 64 ; i++) cusage[i] /= aa_toto ;
+	return aa_toto ;
+}
+
+
+/* loads a codon usage table from file */
+int	gp_codon_load_usage(FILE *fp, double utable[64]) {
+	int line = 0, i, index ;
+	char bufor[BUFSIZ], codon[4] ;
+	double usage ;
+
+	/* zeroing the codon usage table */
+  for(i = 0 ; i < 64 ; i++) utable[i] = 0.0 ;
+
+	/* read the file */
+	while(fgets(bufor, BUFSIZ, fp) != NULL) {
+		line++ ;
+
+		if(*bufor == '#' || *bufor == '\n') continue ;
+
+		/* trying to read the line */
+		if(sscanf(bufor, "%3s %lf", codon, &usage) != 2) {
+			gp_warn("Problems reading line %i of the codon table", line) ;
+			continue ;
+		}
+
+		/* converting codon into 3d coordinates */
+		if( (index = gp_codon_get_index(codon)) < 0) {
+			gp_warn("Bad codon %s, line %i of the codon table", codon, line) ;
+			continue ;
+		}
+
+		utable[index] = usage ; 
+	}
+
+	return EXIT_SUCCESS ;
+}
+
+
+/* load a codon table from a file */
+int gp_codon_load_code(FILE *fp, int ctable[64]) {
+	char bufor[BUFSIZ], codon[4] = "\0\0\0\0", c ;
+	int line = 0, index ;
+
+	while( (fgets(bufor, BUFSIZ, fp)) ){
+		line++ ;
+		if(*bufor == '#' || *bufor == '\n') continue ;
+
+		if(sscanf(bufor, "%3s %c", codon, &c) != 2) {
+			gp_warn("Problem reading codon table line %i", line) ;
+			continue ;
+		}
+
+		c = toupper(c) ;
+	
+		if( (index = gp_codon_get_index(codon)) < 0) {
+			gp_warn("Bad codon in codon table line %i", line) ;
+			continue ;
+		}
+
+		if(debug) gp_warn("read codon %s [%i] -> aa %c", codon, index, c) ;
+		ctable[index] = c ;
+	}
+
+	return EXIT_SUCCESS ;
+}
+
+
+/* given an index, returns the respective codon */
+int gp_codon_get_codon(char *codon, int index) {
+	char cnv[4] = { 'A', 'C', 'G', 'T' } ;
+	int x, y, z ;
+
+	if(index > 63 || index < 0) return EXIT_FAILURE ;
+
+	x = index / 16 ;
+	codon[0] = cnv[x] ;
+	index -= x * 16 ;
+	y = index / 4 ;
+	codon[1] = cnv[y] ;
+	index -= y * 4 ;
+	z = index ;
+	codon[2] = cnv[z] ;
+	codon[3] = '\0' ;
+
+	return EXIT_SUCCESS ;
+}
+
+
+/* given a codon, returns it index (0 for AAA, 1 for AAC... ....63 for UUU */
+int	gp_codon_get_index(char *codon) {
+	char c ;
+	int i, tmp[3], res = -1 ;
+
+	if(!codon) return res ;
+
+	for(i = 0 ; i < 3 ; i++) {
+		c = toupper(codon[i]) ;
+	
+		switch(c){
+			case 'A': tmp[i] = 0 ;
+				break ;
+			case 'C': tmp[i] = 1 ;
+				break ;
+			case 'G': tmp[i] = 2 ;
+				break ;
+			case 'T':
+			case 'U': tmp[i] = 3 ;
+				break ;
+			default: /* not a nucleotide */
+				return res ;
+				break ;
+		}
+	}
+
+	res = tmp[0] * 16 + tmp[1] * 4 + tmp[2] ;
+	return res ;
+}
+
+
+/* load the standard genetic code into the specified codon table */
+void gp_codon_load_code_standard(int ctable[64]) {
+
+	ctable[0] = 'K' ; ctable[1] = 'N' ; ctable[2] = 'K' ; ctable[3] = 'N' ;
+	ctable[4] = 'T' ; ctable[5] = 'T' ; ctable[6] = 'T' ; ctable[7] = 'T' ;
+	ctable[8] = 'R' ; ctable[9] = 'S' ; ctable[10] = 'R' ; ctable[11] = 'S' ;
+	ctable[12] = 'I' ; ctable[13] = 'I' ; ctable[14] = 'M' ; ctable[15] = 'I' ;
+	ctable[16] = 'Q' ; ctable[17] = 'H' ; ctable[18] = 'Q' ; ctable[19] = 'H' ;
+	ctable[20] = 'P' ; ctable[21] = 'P' ; ctable[22] = 'P' ; ctable[23] = 'P' ;
+	ctable[24] = 'R' ; ctable[25] = 'R' ; ctable[26] = 'R' ; ctable[27] = 'R' ;
+	ctable[28] = 'L' ; ctable[29] = 'L' ; ctable[30] = 'L' ; ctable[31] = 'L' ;
+	ctable[32] = 'E' ; ctable[33] = 'D' ; ctable[34] = 'E' ; ctable[35] = 'D' ;
+	ctable[36] = 'A' ; ctable[37] = 'A' ; ctable[38] = 'A' ; ctable[39] = 'A' ;
+	ctable[40] = 'G' ; ctable[41] = 'G' ; ctable[42] = 'G' ; ctable[43] = 'G' ;
+	ctable[44] = 'V' ; ctable[45] = 'V' ; ctable[46] = 'V' ; ctable[47] = 'V' ;
+	ctable[48] = '0' ; ctable[49] = 'T' ; ctable[50] = '0' ; ctable[51] = 'T' ;
+	ctable[52] = 'S' ; ctable[53] = 'S' ; ctable[54] = 'S' ; ctable[55] = 'S' ;
+	ctable[56] = '0' ; ctable[57] = 'C' ; ctable[58] = 'W' ; ctable[59] = 'C' ;
+	ctable[60] = 'L' ; ctable[61] = 'F' ; ctable[62] = 'L' ; ctable[63] = 'F' ;
+}
+
 /* 
  * The standard codon table is loaded. In Genpak programs, A= 0, C= 1, G= 2
  * and T/U= 3
+ * This function is depreciated. Use gp_codon_load_code_standard instead 
  */
 
 codont * gp_codon_table_load_std() {
@@ -1040,7 +1210,10 @@ codont * gp_codon_table_load_std() {
  */
 
 void gp_codon_init_conversion() {
-
+	int i = 0 ;
+	
+	/* in case smth goes wrong... */
+	for(i = 0 ; i<128 ; i++) *one2three[i] = '\0' ;
 
 	strcpy(one2three['0'],"STP") ;
 	strcpy(one2three['G'],"Gly") ;
