@@ -204,25 +204,22 @@ double dimer_find(sekw *s, opt_s *o) {
 
 	dimeres = malloc( (s->leng - 2) * sizeof(*dimeres) ) ;
 
-	for(shift = r->sequ, i = 0 ; 
-		shift < (r->sequ + r->leng - 2) ; shift++, i++) {
+	for(shift = s->sequ, i = 0 ; shift < (s->sequ + s->leng - 2) ; shift++, i++) {
 		
 		dimeres[i].energy = 0 ;
 		dimeres[i].shift = i ;
 		last = FALSE ;
 
-		for(p = shift, q = s->sequ ; 
-			p < (r->sequ + r->leng - 1); p++, q++) {
+		for(p = shift, q = r->sequ ; p < (s->sequ + s->leng - 1) ; p++) {
 
 			if(Compare(*p, *q)) {
 				if(last)
 					dimeres[i].energy += dg[cnv[*(p - 1)]][cnv[*p]] ;
 				last = TRUE ;
-			} else {
-				last = FALSE ;
-			}
+			} else last = FALSE ;
 
 		}
+
 	}
 
 	qsort(dimeres, (s->leng - 2), sizeof(*dimeres), stem_compare) ;
@@ -248,17 +245,15 @@ int dimer_print(sekw *s, stem_s *d, opt_s *o) {
 	int i ;
 	char c, *p ;
 
-	if(d->energy > -0.1) return EXIT_SUCCESS ;
-
 	fprintf(o->out, "dG %.2f kcal/mol\n", d->energy) ;
 	for(i = 0 ; i < d->shift ; i++) fprintf(o->out, " ") ;
 
 	fprintf(o->out, "5'-%s-3'\n   ", s->sequ) ;
 
 	for(i = 0 ; i < d->shift ; i++) fprintf(o->out, " ") ;
-	for(i = 0 ; i < (s->leng - d->shift) ; i++) {
+	for(i = d->shift ; i < (s->leng) ; i++) {
 
-		c = gp_nucl_complement(s->sequ[s->leng - i - d->shift - 1 ]) ;
+		c = gp_nucl_complement(s->sequ[s->leng - i + d->shift - 1]) ;
 		if(Compare(c, s->sequ[i])) fprintf(o->out, "|") ;
 		else fprintf(o->out, " ") ;
 
@@ -331,9 +326,8 @@ int stem_compare(const void *s1, const void *s2) {
 	a = (stem_s *) s1 ;
 	b = (stem_s *) s2 ;
 
-	if(a->energy > b->energy) return 1 ;
-	if(a->energy < b->energy) return -1 ;
-	return 0 ;
+	return (a->energy - b->energy) ;
+
 }
 
 
@@ -404,8 +398,6 @@ int stem_print(sekw *s, stem_s *stem, opt_s *o) {
 	int l, ls, i, j, k ;
 	char c, *p, *q, sp[101] ;
 
-	if(stem->energy > -0.1) return EXIT_SUCCESS ;
-
 	/* checking whether parameters OK */
 	if( stem->start < s->sequ || 
 		stem->end > (s->sequ + s->leng) || 
