@@ -81,6 +81,7 @@ int main(int argc, char *argv[])
 	opt.thumb = FALSE ;
 	opt.ions = 1 ;
 	opt.mols = 0.0001 ;
+	opt.name = FALSE ;
 
 	while ((c = gp_getopt(argc, argv, "NtsM:c:mvHhqd")) != EOF)
 		switch(c) {
@@ -160,11 +161,11 @@ int main(int argc, char *argv[])
 		} else {
 			dh = CompParam(inseq, DH, opt) ;
 			ds = CompParam(inseq, DS, opt) ;
-			if(debug) gp_warn("dg: %f ds: %f\n", dh, ds) ;
 			tm = dh * 1000 / (ds + R * log(opt.mols)) + absZero ;
 			/* Salt correction */
 			tm = tm + 12.5 * log10(opt.ions) ;
-			if(tm < 0.0) tm = 0.0 ;
+			if(debug) gp_warn("dS = %f, dH = %f", ds, dh) ;
+			if(tm < 1.0) tm = 0.0 ;
 		}
 
 		fprintf(out, "%.1f ", tm) ;
